/*
 * Copyright (c) ROSA Laboratory 2011
 * License: GPLv3
 *
 * Authors:
 * Alexey Yermakov <alexey.yermakov@rosalab.ru>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3,
 *   or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef __lnswidget_h_
#define __lnswidget_h_

#include <QGraphicsItem>
#include <QStyleOptionGraphicsItem>
#include <QGraphicsWidget>
#include <QWidget>
#include <QString>
#include <QObject>
#include <QPixmap>
#include <QGraphicsSceneMouseEvent>


#include <Plasma/Applet>

#include "config.h"

class LNSWidget : public QGraphicsWidget
{
  Q_OBJECT
    public:
  LNSWidget(QGraphicsItem *parent = 0);
  virtual ~LNSWidget(void);

  enum ButtonState
  {
    BS_DEFAULT,
    BS_DISABLE,
    BS_HOVER,
    BS_PRESS
  };

  QRectF boundingRect() const;
  void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);

 signals:
  void lockButtonClicked(void);
  void offButtonClicked(void);
  
 protected:
  void mousePressEvent(QGraphicsSceneMouseEvent *event);
  void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);
  void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
  void hoverMoveEvent(QGraphicsSceneHoverEvent *event);
  void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
  
 private:
  typedef QMap<QString, QPixmap*> PixmapMap;
  PixmapMap m_pixmaps;
  
  void loadPixmaps(void);
  void unloadPixmaps(void);

  QSize m_backgroundSize;

  ButtonState m_lockButtonState;
  ButtonState m_offButtonState;

  QRect m_lockButtonRect;
  QRect m_offButtonRect;
};

#endif // __lnswidget_h_
