/*
 * Copyright (c) ROSA Laboratory 2011
 * License: GPLv3
 *
 * Authors:
 *  Julia Mineeva <julia.mineeva@osinit.ru>
 *  Evgeniy Augin <evgeniy.augin@osinit.ru>
 *  Ivan Kulkov   <ivan.kulkov@osinit.ru>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3,
 *   or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#include "videoeventwidget.h"
#include "activityeventssource.h"

#include <QStringList>
#include <QPainter>
#include <QtSvg/QSvgRenderer>

#include <KIcon>

#include <kio/previewjob.h>
#include <kio/filejob.h>
#include <kurl.h>
#include <kfileitem.h>
#include <klocale.h>


QVideoEventWidget::QVideoEventWidget(QGraphicsItem* parent, Qt::WindowFlags wFlags)
  : QActivityEventWidget(parent, wFlags)
{
    m_playPic.load(":/icons/pla-empty-str.png");
    /*
    QPixmap transparent(m_playPic.size());
    // Do transparency
    transparent.fill(Qt::transparent);
    QPainter p(&transparent);
    p.setCompositionMode(QPainter::CompositionMode_Source);
    p.drawPixmap(0, 0, m_playPic);
    p.setCompositionMode(QPainter::CompositionMode_DestinationIn);
    p.fillRect(transparent.rect(), QColor(0, 0, 0, 150));
    p.end();

    m_playPic = transparent;
    */
    setContentsMargins(5,5,5,5);
    m_image.load(":/icons/pla-empty-box.png");
}

QVideoEventWidget::~QVideoEventWidget()
{
}

void QVideoEventWidget::paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget)
{
    QRectF g = contentsRect();
    QImage image;
    if (!m_image.isNull())
    {
        image = m_image.scaled(QSize(g.width(), g.height()), Qt::KeepAspectRatio, Qt::SmoothTransformation);
        painter->drawImage(g.left() + (g.width() - image.width())/2, g.top() + (g.height() - image.height())/2, image);
    }
    QActivityEventWidget::paint(painter,option,widget);
}

void QVideoEventWidget::getEventThumb(ActivityEvent* event)
{
    KUrl item(QString::fromLocal8Bit(event->url().data()));
    KFileItem kitem(KFileItem::Unknown, KFileItem::Unknown, item, false);
    KFileItemList klist;
    klist.append(kitem);
    QStringList list = KIO::PreviewJob::availablePlugins();
    KIO::PreviewJob* job = KIO::filePreview(klist, 256,0 , 0, 0, true, true, &list);

    connect(job, SIGNAL(gotPreview(const KFileItem&, const QPixmap&)),
            this, SLOT(showPreview(const KFileItem&, const QPixmap&)));
    connect(job, SIGNAL(failed(const KFileItem&)), this, SLOT(setNullIcon(const KFileItem&)));
}

void QVideoEventWidget::init(ActivityEvent *event, const QRectF &widgetRect)
{
  QActivityEventWidget::init(event, widgetRect);

  if (!event) return;

  QString ttText = i18n("%1\nrating: %2", QString::fromLocal8Bit(event->text().c_str()), event->rate());
  setToolTip(ttText);

  getEventThumb(event);
}

void QVideoEventWidget::showPreview(const KFileItem&, const QPixmap& pix)
{
  QPixmap image = pix;
  //m_playPic = m_playPic.scaled(pix.width()/3,pix.height()/3,Qt::KeepAspectRatio);
  //QImage im;
  QSvgRenderer svgr(QString(":/icons/pla-empty-str.svg"));
  //QPainter p0(&im);

  //p0.setRenderHint(QPainter::Antialiasing, false);
  //svgr.render(&p0);
 // p0.end();
  //im = im.scaled(pix.width()/3,pix.height()/3,Qt::KeepAspectRatio);

  QPainter p(&image);
  //p.drawPixmap((image.width() - m_playPic.width())/2,(image.height() - m_playPic.height())/2, m_playPic);
  //p.drawImage((image.width() - im.width())/2,(image.height() - im.height())/2,im);
  QRectF rect(image.width()/3,(image.height()-(image.width()/3))/2,image.width()/3,image.width()/3);
  svgr.render(&p,rect);
  //p0.end();
  p.end();
  m_image = image.toImage();
  update();
}

void QVideoEventWidget::setNullIcon(const KFileItem& item)
{
    KIcon icon(item.iconName(), 0, item.overlays());
    m_image = icon.pixmap(256).toImage();
    update();
}
